@@ -10,7 +10,7 @@ from ..owl_literal import OWLLiteral
 from ..owl_individual import OWLIndividual
 from ..owl_datatype import OWLDatatype
 from ..owl_object import OWLObject
-from owlapy.vocab import OWLFacet
+from ..vocab import OWLFacet
 from datetime import datetime, date
 from pandas import Timedelta
 
@@ -53,7 +53,7 @@ class OWLHasValueRestriction(Generic[_T], OWLRestriction, HasFiller[_T], metacla
     """Represent a HasValue restriction in the OWL 2
 
     Args:
-        _T: The value type.
+        value: The value type _T.
     """
     __slots__ = ()
 
@@ -65,7 +65,7 @@ class OWLHasValueRestriction(Generic[_T], OWLRestriction, HasFiller[_T], metacla
     def __eq__(self, other):
         if type(other) is type(self):
             return self._v == other._v
-        return NotImplemented
+        return False
 
     def __hash__(self):
         return hash(self._v)
@@ -143,11 +143,13 @@ class OWLQuantifiedObjectRestriction(OWLQuantifiedRestriction[OWLClassExpression
         return self._filler
 
 
-class OWLObjectCardinalityRestriction(OWLCardinalityRestriction[OWLClassExpression], OWLQuantifiedObjectRestriction):
+class OWLObjectCardinalityRestriction(OWLCardinalityRestriction[OWLClassExpression], OWLQuantifiedObjectRestriction,
+                                      metaclass=ABCMeta):
     """Represents Object Property Cardinality Restrictions in the OWL 2 specification."""
     __slots__ = ()
 
     _property: OWLObjectPropertyExpression
+    # @TODO: CD: property shows the in-built function
 
     @abstractmethod
     def __init__(self, cardinality: int, property: OWLObjectPropertyExpression, filler: OWLClassExpression):
@@ -163,11 +165,12 @@ class OWLObjectCardinalityRestriction(OWLCardinalityRestriction[OWLClassExpressi
                f"property={repr(self.get_property())},{self.get_cardinality()},filler={repr(self.get_filler())})"
 
     def __eq__(self, other):
-        if type(other) == type(self):
+        if type(other) is type(self):
             return self._property == other._property \
                 and self._cardinality == other._cardinality \
                 and self._filler == other._filler
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
         return hash((self._property, self._cardinality, self._filler))
@@ -272,10 +275,11 @@ class OWLObjectSomeValuesFrom(OWLQuantifiedObjectRestriction):
     def __eq__(self, other):
         if type(other) is type(self):
             return self._filler == other._filler and self._property == other._property
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash((self._filler, self._property))
+        return hash(("OWLObjectSomeValuesFrom",self._filler, self._property))
 
     def get_property(self) -> OWLObjectPropertyExpression:
         # documented in parent
@@ -288,7 +292,7 @@ class OWLObjectAllValuesFrom(OWLQuantifiedObjectRestriction):
     individuals that are instances of CE. (https://www.w3.org/TR/owl2-syntax/#Universal_Quantification)"""
     __slots__ = '_property', '_filler'
     type_index: Final = 3006
-
+    # @TODO: CD: property shows the in-built function
     def __init__(self, property: OWLObjectPropertyExpression, filler: OWLClassExpression):
         super().__init__(filler)
         self._property = property
@@ -299,10 +303,11 @@ class OWLObjectAllValuesFrom(OWLQuantifiedObjectRestriction):
     def __eq__(self, other):
         if type(other) is type(self):
             return self._filler == other._filler and self._property == other._property
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash((self._filler, self._property))
+        return hash(("OWLObjectAllValuesFrom",self._filler, self._property))
 
     def get_property(self) -> OWLObjectPropertyExpression:
         # documented in parent
@@ -335,12 +340,13 @@ class OWLObjectHasSelf(OWLObjectRestriction):
         return self._property
 
     def __eq__(self, other):
-        if type(other) == type(self):
+        if type(other) is type(self):
             return self._property == other._property
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash(self._property)
+        return hash(("OWLObjectHasSelf", self._property))
 
     def __repr__(self):
         return f'OWLObjectHasSelf({self._property})'
@@ -357,7 +363,7 @@ class OWLObjectHasValue(OWLHasValueRestriction[OWLIndividual], OWLObjectRestrict
 
     _property: OWLObjectPropertyExpression
     _v: OWLIndividual
-
+    # @TODO: CD: property shows the in-built function
     def __init__(self, property: OWLObjectPropertyExpression, individual: OWLIndividual):
         """
         Args:
@@ -393,13 +399,15 @@ class OWLObjectOneOf(OWLAnonymousClassExpression, HasOperands[OWLIndividual]):
     __slots__ = '_values'
     type_index: Final = 3004
 
-    def __init__(self, values: Union[OWLIndividual, Iterable[OWLIndividual]]):
+    def __init__(self, values: OWLIndividual | Iterable[OWLIndividual]):
+        #assert isinstance(values, OWLIndividual) | isinstance(values, set)
+        #    f"The input of OWLObjectOneOf must be either an OWLIndividual or a set of OWLIndividual. Currently, {type(values)}!"
         if isinstance(values, OWLIndividual):
             self._values = values,
         else:
             for _ in values:
                 assert isinstance(_, OWLIndividual)
-            self._values = tuple(values)
+            self._values = frozenset(values)
 
     def individuals(self) -> Iterable[OWLIndividual]:
         """Gets the individuals that are in the oneOf. These individuals represent the exact instances (extension)
@@ -426,12 +434,15 @@ class OWLObjectOneOf(OWLAnonymousClassExpression, HasOperands[OWLIndividual]):
         return OWLObjectUnionOf(map(lambda _: OWLObjectOneOf(_), self.individuals()))
 
     def __hash__(self):
-        return hash(self._values)
+        return hash(("OWLObjectOneOf", self._values))
 
     def __eq__(self, other):
-        if type(other) == type(self):
-            return self._values == other._values
-        return NotImplemented
+        if type(other) is type(self):
+            if isinstance(self._values, OWLIndividual):
+                return self._values == other._values
+            else:
+                return set(self._values) == set(other._values) and len(self._values) == len(other._values)
+        return False
 
     def __repr__(self):
         return f'OWLObjectOneOf({self._values})'
@@ -450,8 +461,6 @@ class OWLDataRestriction(OWLRestriction, metaclass=ABCMeta):
         # documented in parent
         return True
 
-    pass
-
 
 class OWLQuantifiedDataRestriction(OWLQuantifiedRestriction[OWLDataRange],
                                    OWLDataRestriction, metaclass=ABCMeta):
@@ -461,7 +470,9 @@ class OWLQuantifiedDataRestriction(OWLQuantifiedRestriction[OWLDataRange],
     _filler: OWLDataRange
 
     def __init__(self, filler: OWLDataRange):
+        assert isinstance(filler, OWLDataRange), "filler must be an OWLDataRange"
         self._filler = filler
+    # @TODO:CD: define it as @property
 
     def get_filler(self) -> OWLDataRange:
         # documented in parent (HasFiller)
@@ -475,29 +486,31 @@ class OWLDataCardinalityRestriction(OWLCardinalityRestriction[OWLDataRange],
     __slots__ = ()
 
     _property: OWLDataPropertyExpression
+    # @TODO: CD: property shows the in-built function
 
     @abstractmethod
     def __init__(self, cardinality: int, property: OWLDataPropertyExpression, filler: OWLDataRange):
+        assert isinstance(filler, OWLDataRange), "filler must be an OWLDataRange"
         super().__init__(cardinality, filler)
         self._property = property
 
-    def get_property(self) -> OWLDataPropertyExpression:
-        # documented in parent
-        return self._property
+    def __hash__(self):
+        return hash(("OWLDataCardinalityRestriction",self._property, self._cardinality, self._filler))
 
     def __repr__(self):
         return f"{type(self).__name__}(" \
                f"property={repr(self.get_property())},{self.get_cardinality()},filler={repr(self.get_filler())})"
 
     def __eq__(self, other):
-        if type(other) == type(self):
-            return self._property == other._property \
-                and self._cardinality == other._cardinality \
-                and self._filler == other._filler
-        return NotImplemented
+        if type(other) is type(self):
+            return (self._property == other._property and self._cardinality == other._cardinality
+                    and self._filler == other._filler)
+        else:
+            return False
 
-    def __hash__(self):
-        return hash((self._property, self._cardinality, self._filler))
+    def get_property(self) -> OWLDataPropertyExpression:
+        # documented in parent
+        return self._property
 
 
 class OWLDataMinCardinality(OWLDataCardinalityRestriction):
@@ -509,6 +522,7 @@ class OWLDataMinCardinality(OWLDataCardinalityRestriction):
     __slots__ = '_cardinality', '_filler', '_property'
 
     type_index: Final = 3015
+    # @TODO: CD: property shows the in-built function
 
     def __init__(self, cardinality: int, property: OWLDataPropertyExpression, filler: OWLDataRange):
         """
@@ -532,6 +546,7 @@ class OWLDataMaxCardinality(OWLDataCardinalityRestriction):
     __slots__ = '_cardinality', '_filler', '_property'
 
     type_index: Final = 3017
+    # @TODO: CD: property shows the in-built function
 
     def __init__(self, cardinality: int, property: OWLDataPropertyExpression, filler: OWLDataRange):
         """
@@ -555,6 +570,7 @@ class OWLDataExactCardinality(OWLDataCardinalityRestriction):
     __slots__ = '_cardinality', '_filler', '_property'
 
     type_index: Final = 3016
+    # @TODO: CD: property shows the in-built function
 
     def __init__(self, cardinality: int, property: OWLDataPropertyExpression, filler: OWLDataRange):
         """
@@ -591,6 +607,7 @@ class OWLDataSomeValuesFrom(OWLQuantifiedDataRestriction):
     type_index: Final = 3012
 
     _property: OWLDataPropertyExpression
+    # @TODO: CD: property shows the in-built function
 
     def __init__(self, property: OWLDataPropertyExpression, filler: OWLDataRange):
         """Gets an OWLDataSomeValuesFrom restriction.
@@ -611,10 +628,11 @@ class OWLDataSomeValuesFrom(OWLQuantifiedDataRestriction):
     def __eq__(self, other):
         if type(other) is type(self):
             return self._filler == other._filler and self._property == other._property
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash((self._filler, self._property))
+        return hash(("OWLDataSomeValuesFrom",self._filler, self._property))
 
     def get_property(self) -> OWLDataPropertyExpression:
         # documented in parent
@@ -634,6 +652,7 @@ class OWLDataAllValuesFrom(OWLQuantifiedDataRestriction):
     type_index: Final = 3013
 
     _property: OWLDataPropertyExpression
+    # @TODO:CD:property shows the in-built function
 
     def __init__(self, property: OWLDataPropertyExpression, filler: OWLDataRange):
         """Gets an OWLDataAllValuesFrom restriction.
@@ -654,10 +673,11 @@ class OWLDataAllValuesFrom(OWLQuantifiedDataRestriction):
     def __eq__(self, other):
         if type(other) is type(self):
             return self._filler == other._filler and self._property == other._property
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash((self._filler, self._property))
+        return hash(("OWLDataAllValuesFrom",self._filler, self._property))
 
     def get_property(self) -> OWLDataPropertyExpression:
         # documented in parent
@@ -671,17 +691,22 @@ class OWLDataHasValue(OWLHasValueRestriction[OWLLiteral], OWLDataRestriction):
     (https://www.w3.org/TR/owl2-syntax/#Literal_Value_Restriction)
     """
     __slots__ = '_property'
+    # @TODO:CD:What is _v? even if it is inherited from somewhere, we should add it into docstring.
+    # @TODO:CD: We should also name the class attributes in a more meaningful manner.
+    # AB re: _v is indeed inherited and represents the value of this DataHasValue CE. It's name is kept general
+    # because the value can have different types depending on the implementor of the base class.
 
     type_index: Final = 3014
 
     _property: OWLDataPropertyExpression
+    # @TODO: CD: property shows the in-built function
 
     def __init__(self, property: OWLDataPropertyExpression, value: OWLLiteral):
         """Gets an OWLDataHasValue restriction.
 
         Args:
             property: The data property that the restriction acts along.
-            filler: The literal value.
+            value: The literal value.
 
         Returns:
             An OWLDataHasValue restriction along the specified property with the specified literal.
@@ -695,10 +720,11 @@ class OWLDataHasValue(OWLHasValueRestriction[OWLLiteral], OWLDataRestriction):
     def __eq__(self, other):
         if type(other) is type(self):
             return self._v == other._v and self._property == other._property
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash((self._v, self._property))
+        return hash(("OWLDataHasValue", self._v, self._property))
 
     def as_some_values_from(self) -> OWLClassExpression:
         """A convenience method that obtains this restriction as an existential restriction with a nominal filler.
@@ -728,6 +754,19 @@ class OWLDataOneOf(OWLDataRange, HasOperands[OWLLiteral]):
                 assert isinstance(_, OWLLiteral)
             self._values = tuple(values)
 
+    def __repr__(self):
+        return f'OWLDataOneOf({self._values})'
+
+    def __hash__(self):
+        return hash(("OWLDataOneOf",self._values))
+
+    def __eq__(self, other):
+        if type(other) is type(self):
+            return {i for i in self._values} == {j for j in other._values}
+        else:
+            return False
+    # TODO:CD: define it as @property as the name of the class method does not correspond to an action
+
     def values(self) -> Iterable[OWLLiteral]:
         """Gets the values that are in the oneOf.
 
@@ -740,17 +779,6 @@ class OWLDataOneOf(OWLDataRange, HasOperands[OWLLiteral]):
         # documented in parent
         yield from self.values()
 
-    def __hash__(self):
-        return hash(self._values)
-
-    def __eq__(self, other):
-        if type(other) == type(self):
-            return self._values == other._values
-        return NotImplemented
-
-    def __repr__(self):
-        return f'OWLDataOneOf({self._values})'
-
 
 class OWLDatatypeRestriction(OWLDataRange):
     """A datatype restriction DatatypeRestriction( DT F1 lt1 ... Fn ltn ) consists of a unary datatype DT and n pairs
@@ -767,7 +795,7 @@ class OWLDatatypeRestriction(OWLDataRange):
     _facet_restrictions: Sequence['OWLFacetRestriction']
 
     def __init__(self, type_: OWLDatatype, facet_restrictions: Union['OWLFacetRestriction',
-    Iterable['OWLFacetRestriction']]):
+                                                                     Iterable['OWLFacetRestriction']]):
         self._type = type_
         if isinstance(facet_restrictions, OWLFacetRestriction):
             facet_restrictions = facet_restrictions,
@@ -783,10 +811,11 @@ class OWLDatatypeRestriction(OWLDataRange):
         if type(other) is type(self):
             return self._type == other._type \
                 and self._facet_restrictions == other._facet_restrictions
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash((self._type, self._facet_restrictions))
+        return hash(("OWLDatatypeRestriction", self._type, self._facet_restrictions))
 
     def __repr__(self):
         return f'OWLDatatypeRestriction({repr(self._type)}, {repr(self._facet_restrictions)})'
@@ -818,10 +847,11 @@ class OWLFacetRestriction(OWLObject):
     def __eq__(self, other):
         if type(other) is type(self):
             return self._facet == other._facet and self._literal == other._literal
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
-        return hash((self._facet, self._literal))
+        return hash(("OWLFacetRestriction", self._facet, self._literal))
 
     def __repr__(self):
         return f'OWLFacetRestriction({self._facet}, {repr(self._literal)})'
