@@ -85,7 +85,7 @@ class VariablesMapping:
 class Owl2SparqlConverter:
     """Convert owl (owlapy model class expressions) to SPARQL."""
     __slots__ = 'ce', 'sparql', 'variables', 'parent', 'parent_var', 'properties', 'variable_entities', 'cnt', \
-                'mapping', 'grouping_vars', 'having_conditions', '_intersection'
+                'mapping', 'grouping_vars', 'having_conditions', 'for_all_de_morgan', 'named_individuals', '_intersection'
     # @TODO:CD: We need to document this class. The computation behind the mapping is not clear.
 
     ce: OWLClassExpression
@@ -100,8 +100,13 @@ class Owl2SparqlConverter:
     grouping_vars: Dict[OWLClassExpression, Set[str]]
     having_conditions: Dict[OWLClassExpression, Set[str]]
     cnt: int
+    for_all_de_morgan: bool
+    named_individuals: bool
 
-    def convert(self, root_variable: str, ce: OWLClassExpression, named_individuals: bool = False):
+    def convert(self, root_variable: str,
+                ce: OWLClassExpression,
+                for_all_de_morgan: bool = True,
+                named_individuals: bool = False):
         """Used to convert owl class expression to SPARQL syntax.
 
         Args:
@@ -124,9 +129,11 @@ class Owl2SparqlConverter:
         self.mapping = VariablesMapping()
         self.grouping_vars = defaultdict(set)
         self.having_conditions = defaultdict(set)
-        # if named_individuals is True, we return only entities that are instances of owl:NamedIndividual
-        if named_individuals:
-            self.append_triple(root_variable, 'a', f"<{OWLRDFVocabulary.OWL_NAMED_INDIVIDUAL.as_str()}>")
+        self.for_all_de_morgan = for_all_de_morgan
+        self.named_individuals = named_individuals
+        # # if named_individuals is True, we return only entities that are instances of owl:NamedIndividual
+        # if named_individuals:
+        #     self.append_triple(root_variable, 'a', f"<{OWLRDFVocabulary.OWL_NAMED_INDIVIDUAL.as_str()}>")
         with self.stack_variable(root_variable):
             with self.stack_parent(ce):
                 self.process(ce)
@@ -204,7 +211,7 @@ class Owl2SparqlConverter:
     # each overload of the method is responsible for processing a different type of class expressions (e.g., ⊔ or ⊓)
     @singledispatchmethod
     def process(self, ce: OWLClassExpression):
-        raise NotImplementedError(ce)
+        raise NotImplementedError(f"We cannot create SPARQL query based on the following owl class {ce}")
 
     # an overload of process function
     # this overload is responsible for handling single concepts (e.g., Brother)
@@ -255,7 +262,19 @@ class Owl2SparqlConverter:
     @process.register
     def _(self, ce: OWLObjectComplementOf):
         subject = self.current_variable
-        self.append_triple(subject, self.mapping.new_individual_variable(), self.mapping.new_individual_variable())
+        # the conversion was trying here to optimize the query
+        # but the proposed optimization alters the semantics of some queries
+        # example: ( A ⊓ ( B ⊔ ( ¬C ) ) )
+        # with the proposed optimization, the group graph pattern for (¬C) will be { FILTER NOT EXISTS { ?x a C } }
+        # however, the expected pattern is { ?x ?p ?o . FILTER NOT EXISTS { ?x a C } }
+        # the exclusion of "?x ?p ?o" results in the group graph pattern to just return true or false (not bindings)
+        # as a result, we need to comment out the if-clause of the following line
+        # if not self.in_intersection and self.modal_depth == 1:
+        # if namedIndividual is set to True, do not use variables --> restrict the subject to instances of NamedIndividual
+        if self.named_individuals:
+            self.append_triple(subject, "a", f"<{OWLRDFVocabulary.OWL_NAMED_INDIVIDUAL.as_str()}>")
+        else:
+            self.append_triple(subject, self.mapping.new_individual_variable(), self.mapping.new_individual_variable())
 
         self.append("FILTER NOT EXISTS { ")
         # process the concept after the ¬
@@ -280,11 +299,17 @@ class Owl2SparqlConverter:
         with self.stack_variable(object_variable):
             self.process(filler)
 
+    @process.register
+    def _(self, ce: OWLObjectAllValuesFrom):
+        if self.for_all_de_morgan is True:
+            self.forAllDeMorgan(ce)
+        else:
+            self.forAll(ce)
+
     # an overload of process function
     # this overload is responsible for handling the forAll operator (e.g., ∀hasChild.Male)
     # general case: ∀r.C
-    @process.register
-    def _(self, ce: OWLObjectAllValuesFrom):
+    def forAll(self, ce: OWLObjectAllValuesFrom):
         subject = self.current_variable
         object_variable = self.mapping.new_individual_variable()
         # property expression holds the role of the class expression (hasChild in our example)
@@ -333,6 +358,34 @@ class Owl2SparqlConverter:
             self.append_triple(self.current_variable, predicate, self.mapping.new_individual_variable())
         self.append(" } }")
 
+    # an overload of process function
+    # this overload is responsible for handling the forAll operator but as if the DeMorgan law was applied
+    # (e.g., ∀hasChild.Male == ¬(∃hasChild.¬Male))
+    # general case: ∀r.C == ¬(∃r.¬C)
+    def forAllDeMorgan(self, ce: OWLObjectAllValuesFrom):
+        subject = self.current_variable
+        # here, we need to apply the complement rule twice
+        # the first filter not exists covers the outer ¬
+        self.append_triple(subject, self.mapping.new_individual_variable(), self.mapping.new_individual_variable())
+        self.append("FILTER NOT EXISTS { ")
+        object_variable = self.mapping.new_individual_variable()
+        # property expression holds the role of the class expression (hasChild in our example)
+        property_expression = ce.get_property()
+        if property_expression.is_anonymous():
+            # property expression is inverse of a property
+            self.append_triple(object_variable, property_expression.get_named_property(), self.current_variable)
+        else:
+            self.append_triple(self.current_variable, property_expression.get_named_property(), object_variable)
+
+        # the second filter not exists covers the inner ¬
+        # filler holds the concept of the expression (Male in our example) and is processed recursively
+        self.append("FILTER NOT EXISTS { ")
+        filler = ce.get_filler()
+        with self.stack_variable(object_variable):
+            self.process(filler)
+        self.append(" }")
+        self.append(" }")
+
     # an overload of process function
     # this overload is responsible for handling the exists operator combined with an individual (e.g., ∃hasChild.{john})
     # general case: ∃r.{a}
@@ -455,7 +508,7 @@ class Owl2SparqlConverter:
                 self.append(",")
             assert isinstance(ind, OWLNamedIndividual)
             self.append(f"<{ind.to_string_id()}>")
-        self.append(f" ) )")
+        self.append(" ) )")
 
     @process.register
     def _(self, ce: OWLDataSomeValuesFrom):
@@ -498,7 +551,7 @@ class Owl2SparqlConverter:
     def _(self, ce: OWLDataHasValue):
         property_expression = ce.get_property()
         value = ce.get_filler()
-        assert isinstance(value, OWLDataProperty)
+        assert isinstance(value, OWLLiteral)
         self.append_triple(self.current_variable, property_expression, value)
 
     @process.register
@@ -520,7 +573,7 @@ class Owl2SparqlConverter:
                 self.append(",")
             if value:
                 self.append(self.render(value))
-        self.append(f" ) ) ")
+        self.append(" ) ) ")
 
     @process.register
     def _(self, node: OWLDatatypeRestriction):
@@ -550,21 +603,20 @@ class Owl2SparqlConverter:
     def as_query(self,
                  root_variable: str,
                  ce: OWLClassExpression,
+                 for_all_de_morgan: bool = True,
                  count: bool = False,
                  values: Optional[Iterable[OWLNamedIndividual]] = None,
-                 named_individuals: bool = False)->str:
-        """
-        root variable: the variable that will be projected
-        ce: the class expression to be transformed to a SPARQL query
-        count: True, counts the results ; False, projects the individuals
-        values: positive or negative examples from a class expression problem
-        named_individuals: if set to True, the generated SPARQL query will return only entities that are instances
-        of owl:NamedIndividual
-
-        """
-
+                 named_individuals: bool = False) -> str:
+        assert isinstance(ce,OWLClassExpression), f"ce must be an instance of OWLClassExpression. Currently {type(ce)}"
+        # root variable: the variable that will be projected
+        # ce: the class expression to be transformed to a SPARQL query
+        # for_all_de_morgan: true -> ¬(∃r.¬C), false -> (∀r.C)
+        # count: True, counts the results ; False, projects the individuals
+        # values: positive or negative examples from a class expression problem
+        # named_individuals: if set to True, the generated SPARQL query will return only entities that are instances
+        #                    of owl:NamedIndividual
         qs = ["SELECT"]
-        tp = self.convert(root_variable, ce, named_individuals)
+        tp = self.convert(root_variable, ce, for_all_de_morgan=for_all_de_morgan, named_individuals=named_individuals)
         if count:
             qs.append(f" ( COUNT ( DISTINCT {root_variable} ) AS ?cnt ) WHERE {{ ")
         else:
@@ -573,31 +625,117 @@ class Owl2SparqlConverter:
             q = [f"VALUES {root_variable} {{ "]
             for x in values:
                 q.append(f"<{x.to_string_id()}>")
-            q.append(f"}} . ")
+            q.append("} . ")
             qs.extend(q)
         qs.extend(tp)
-        qs.append(f" }}")
+        qs.append(" }")
 
 
         query = "\n".join(qs)
         parseQuery(query)
         return query
 
+    def as_confusion_matrix_query(self,
+                                  root_variable: str,
+                                  ce: OWLClassExpression,
+                                  positive_examples: Iterable[OWLNamedIndividual],
+                                  negative_examples: Iterable[OWLNamedIndividual],
+                                  for_all_de_morgan: bool = True,
+                                  named_individuals: bool = False) -> str:
+        # get the graph pattern corresponding to the provided class expression (ce)
+        graph_pattern_str = "".join(self.convert(root_variable,
+                                                 ce,
+                                                 for_all_de_morgan=for_all_de_morgan,
+                                                 named_individuals=named_individuals))
+        # preparation for the final query
+
+        # required to compute false negatives
+        number_of_positive_examples = 0
+        # required to compute true negatives
+        number_of_negative_examples = 0
+        # string representation of the positive examples (to be passed to the first VALUES clause)
+        positive_examples_as_str = ""
+        # iterate over the positive examples
+        for positive_example in positive_examples:
+            number_of_positive_examples += 1
+            positive_examples_as_str += f"<{positive_example.to_string_id()}> "
+        assert (len(positive_examples_as_str) > 0)
+
+        # string representation of the positive examples (to be passed to the first VALUES clause)
+        negative_examples_as_str = ""
+        # iterate over the negative examples
+        for negative_example in negative_examples:
+            number_of_negative_examples += 1
+            negative_examples_as_str += f"<{negative_example.to_string_id()}> "
+        assert(len(negative_examples_as_str) > 0)
+
+        # create the sparql query
+        sparql_str= f"""
+                    PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
+                    SELECT * WHERE {{
+                       {{
+                          SELECT (COUNT(DISTINCT {root_variable}) as ?tp) (({number_of_positive_examples} - COUNT(DISTINCT {root_variable})) as ?fn) WHERE {{
+                             VALUES {root_variable} {{ {positive_examples_as_str} }}
+                             {graph_pattern_str}
+                          }}
+                       }}
+                       {{
+                          SELECT (COUNT(DISTINCT {root_variable}) as ?fp) (({number_of_negative_examples} - COUNT(DISTINCT {root_variable})) as ?tn) WHERE {{
+                             VALUES {root_variable} {{ {negative_examples_as_str} }}
+                             {graph_pattern_str}
+                          }}
+                       }}
+                    }}
+                    """
+        parseQuery(sparql_str)
+        return sparql_str
+
 
 converter = Owl2SparqlConverter()
 
 
-def owl_expression_to_sparql(root_variable: str = "?x",
-                             expression: OWLClassExpression = None,
+def owl_expression_to_sparql(expression: OWLClassExpression = None,
+                             root_variable: str = "?x",
                              values: Optional[Iterable[OWLNamedIndividual]] = None,
-                             named_individuals: bool = False)->str:
+                             for_all_de_morgan: bool = True,
+                             named_individuals: bool = False) -> str:
     """Convert an OWL Class Expression (https://www.w3.org/TR/owl2-syntax/#Class_Expressions) into a SPARQL query
      root variable: the variable that will be projected
      expression: the class expression to be transformed to a SPARQL query
 
      values: positive or negative examples from a class expression problem. Unclear
+     for_all_de_morgan: if set to True, the SPARQL mapping will use the mapping containing the nested FILTER NOT EXISTS
+     patterns for the universal quantifier (¬(∃r.¬C)), instead of the counting query
      named_individuals: if set to True, the generated SPARQL query will return only entities
      that are instances of owl:NamedIndividual
     """
     assert expression is not None, "expression cannot be None"
-    return converter.as_query(root_variable, expression, False, values, named_individuals)
+    return converter.as_query(root_variable, expression, count=False, values=values,
+                              named_individuals=named_individuals, for_all_de_morgan=for_all_de_morgan)
+
+
+def owl_expression_to_sparql_with_confusion_matrix(expression: OWLClassExpression,
+                                                   positive_examples: Optional[Iterable[OWLNamedIndividual]],
+                                                   negative_examples: Optional[Iterable[OWLNamedIndividual]],
+                                                   root_variable: str = "?x",
+                                                   for_all_de_morgan: bool = True,
+                                                   named_individuals: bool = False) -> str:
+    """Convert an OWL Class Expression (https://www.w3.org/TR/owl2-syntax/#Class_Expressions) into a SPARQL query
+     root variable: the variable that will be projected
+     expression: the class expression to be transformed to a SPARQL query
+     positive_examples: positive examples from a class expression problem
+     negative_examples: positive examples from a class expression problem
+     for_all_de_morgan: if set to True, the SPARQL mapping will use the mapping containing the nested FILTER NOT EXISTS
+     patterns for the universal quantifier (¬(∃r.¬C)), instead of the counting query
+     named_individuals: if set to True, the generated SPARQL query will return only entities
+     that are instances of owl:NamedIndividual
+    """
+    assert expression is not None, "expression cannot be None"
+    assert positive_examples is not None, "positive examples cannot be None"
+    assert negative_examples is not None, "negative examples cannot be None"
+    return converter.as_confusion_matrix_query(root_variable,
+                                               expression,
+                                               positive_examples=positive_examples,
+                                               negative_examples=negative_examples,
+                                               named_individuals=named_individuals,
+                                               for_all_de_morgan=for_all_de_morgan)
