@@ -2,7 +2,7 @@
 from abc import ABCMeta, abstractmethod
 from itertools import combinations
 
-from typing import TypeVar, List, Optional, Iterable, Generic, Final, Union
+from typing import TypeVar, List, Optional, Iterable, Generic, Union
 from .owl_property import OWLDataPropertyExpression, OWLObjectPropertyExpression
 from .owl_object import OWLObject, OWLEntity
 from .owl_datatype import OWLDatatype, OWLDataRange
@@ -14,9 +14,9 @@ from .iri import IRI
 from owlapy.owl_annotation import OWLAnnotationSubject, OWLAnnotationValue
 from .owl_literal import OWLLiteral
 
-_C = TypeVar('_C', bound='OWLObject')  #:
-_P = TypeVar('_P', bound='OWLPropertyExpression')  #:
-_R = TypeVar('_R', bound='OWLPropertyRange')  #:
+_C = TypeVar('_C', bound='OWLObject')  # noqa: F821
+_P = TypeVar('_P', bound='OWLPropertyExpression')  # noqa: F821
+_R = TypeVar('_R', bound='OWLPropertyRange')  # noqa: F821
 
 
 class OWLAxiom(OWLObject, metaclass=ABCMeta):
@@ -195,9 +195,10 @@ class OWLHasKeyAxiom(OWLLogicalAxiom, HasOperands[OWLPropertyExpression]):
     def __eq__(self, other):
         if type(other) is type(self):
             return self._class_expression == other._class_expression \
-                and self._property_expressions == other._property_expressions \
-                and self._annotations == other._annotations
-        return NotImplemented
+                and set(self._property_expressions) == set(other._property_expressions) \
+                and len(self._property_expressions) == len(other._property_expressions) \
+                and set(self._annotations) == set(other._annotations)
+        return False
 
     def __hash__(self):
         return hash((self._class_expression, *self._property_expressions, *self._annotations))
@@ -257,8 +258,11 @@ class OWLNaryClassAxiom(OWLClassAxiom, OWLNaryAxiom[OWLClassExpression], metacla
 
     def __eq__(self, other):
         if type(other) is type(self):
-            return self._class_expressions == other._class_expressions and self._annotations == other._annotations
-        return NotImplemented
+            # parsed to set to have order-insensitive comparison
+            return (set(self._class_expressions) == set(other._class_expressions)
+                    and len(self._class_expressions) == len(other._class_expressions)
+                    and set(self._annotations) == set(other._annotations))
+        return False
 
     def __hash__(self):
         return hash((*self._class_expressions, *self._annotations))
@@ -281,6 +285,9 @@ class OWLEquivalentClassesAxiom(OWLNaryClassAxiom):
                  annotations: Optional[Iterable['OWLAnnotation']] = None):
         super().__init__(class_expressions=class_expressions, annotations=annotations)
 
+    def __iter__(self):
+        yield from self._class_expressions
+
     def contains_named_equivalent_class(self) -> bool:
         return any(isinstance(ce, OWLClass) for ce in self._class_expressions)
 
@@ -336,8 +343,10 @@ class OWLNaryIndividualAxiom(OWLIndividualAxiom, OWLNaryAxiom[OWLIndividual], me
 
     def __eq__(self, other):
         if type(other) is type(self):
-            return self._individuals == other._individuals and self._annotations == other._annotations
-        return NotImplemented
+            return (set(self._individuals) == set(other._individuals)
+                    and len(self._individuals) == len(other._individuals)
+                    and set(self._annotations) == set(other._annotations))
+        return False
 
     def __hash__(self):
         return hash((*self._individuals, *self._annotations))
@@ -402,8 +411,10 @@ class OWLNaryPropertyAxiom(Generic[_P], OWLPropertyAxiom, OWLNaryAxiom[_P], meta
 
     def __eq__(self, other):
         if type(other) is type(self):
-            return self._properties == other._properties and self._annotations == other._annotations
-        return NotImplemented
+            # parsed to set to have order-insensitive comparison
+            return (set(self._properties) == set(other._properties) and len(self._properties) == len(other._properties)
+                    and set(self._annotations) == set(other._annotations))
+        return False
 
     def __hash__(self):
         return hash((*self._properties, *self._annotations))
@@ -523,6 +534,13 @@ class OWLSubClassOfAxiom(OWLClassAxiom):
         self._super_class = super_class
         super().__init__(annotations=annotations)
 
+    @property
+    def sub_class(self) -> OWLClassExpression:
+        return self._sub_class
+    @property
+    def super_class(self) -> OWLClassExpression:
+        return self._super_class
+
     def get_sub_class(self) -> OWLClassExpression:
         return self._sub_class
 
@@ -533,7 +551,8 @@ class OWLSubClassOfAxiom(OWLClassAxiom):
         if type(other) is type(self):
             return self._super_class == other._super_class and self._sub_class == other._sub_class \
                 and self._annotations == other._annotations
-        return NotImplemented
+        else:
+            return False
 
     def __hash__(self):
         return hash((self._super_class, self._sub_class, *self._annotations))
@@ -576,9 +595,10 @@ class OWLDisjointUnionAxiom(OWLClassAxiom):
 
     def __eq__(self, other):
         if type(other) is type(self):
-            return self._cls == other._cls and self._class_expressions == other._class_expressions \
-                and self._annotations == other._annotations
-        return NotImplemented
+            return (self._cls == other._cls and set(self._class_expressions) == set(other._class_expressions)
+                    and self._annotations == other._annotations
+                    and len(self._class_expressions) == len(other._class_expressions))
+        return False
 
     def __hash__(self):
         return hash((self._cls, *self._class_expressions, *self._annotations))
@@ -723,7 +743,8 @@ class OWLAnnotationAssertionAxiom(OWLAnnotationAxiom):
     _subject: OWLAnnotationSubject
     _annotation: OWLAnnotation
 
-    def __init__(self, subject: OWLAnnotationSubject, annotation: OWLAnnotation):
+    def __init__(self, subject: OWLAnnotationSubject, annotation: OWLAnnotation,
+                 annotations: Optional[Iterable['OWLAnnotation']] = None):
         """Get an annotation assertion axiom - with annotations.
 
         Args:
@@ -732,7 +753,7 @@ class OWLAnnotationAssertionAxiom(OWLAnnotationAxiom):
         """
         assert isinstance(subject, OWLAnnotationSubject)
         assert isinstance(annotation, OWLAnnotation)
-
+        super().__init__(annotations)
         self._subject = subject
         self._annotation = annotation
 
@@ -769,7 +790,7 @@ class OWLAnnotationAssertionAxiom(OWLAnnotationAxiom):
         return hash((self._subject, self._annotation))
 
     def __repr__(self):
-        return f'OWLAnnotationAssertionAxiom({self._subject}, {self._annotation})'
+        return f'OWLAnnotationAssertionAxiom({self._subject}, {self._annotation}, {self.annotations()})'
 class OWLSubAnnotationPropertyOfAxiom(OWLAnnotationAxiom):
     """An annotation subproperty axiom SubAnnotationPropertyOf( AP1 AP2 ) states that the annotation property AP1 is
     a subproperty of the annotation property AP2.
@@ -988,8 +1009,8 @@ class OWLPropertyAssertionAxiom(Generic[_P, _C], OWLIndividualAxiom, metaclass=A
         return hash((self._subject, self._property, self._object, *self._annotations))
 
     def __repr__(self):
-        return f'{type(self).__name__}(subject={self._subject},property={self._property},' \
-               f'object={self._object},annotation={self._annotations})'
+        return f'{type(self).__name__}(subject={self._subject},property_={self._property},' \
+               f'object_={self._object},annotations={self._annotations})'
 
 
 class OWLObjectPropertyAssertionAxiom(OWLPropertyAssertionAxiom[OWLObjectPropertyExpression, OWLIndividual]):
@@ -1028,6 +1049,9 @@ class OWLDataPropertyAssertionAxiom(OWLPropertyAssertionAxiom[OWLDataPropertyExp
 
     def __init__(self, subject: OWLIndividual, property_: OWLDataPropertyExpression, object_: OWLLiteral,
                  annotations: Optional[Iterable['OWLAnnotation']] = None):
+        assert isinstance(subject,OWLIndividual), f"subject must be an OWLIndividual. Currently, {subject} of {type(subject)}"
+        assert isinstance(property_,OWLDataPropertyExpression), f"property_ must be an OWLDataPropertyExpression. Currently, {type(property_)}"
+        assert isinstance(object_,OWLLiteral), f"object_ must be an OWLLiteral. Currently, {type(object_)}"
         super().__init__(subject, property_, object_, annotations)
 
 
@@ -1247,6 +1271,14 @@ class OWLPropertyRangeAxiom(Generic[_P, _R], OWLUnaryPropertyAxiom[_P], metaclas
         self._range = range_
         super().__init__(property_=property_, annotations=annotations)
 
+    @property
+    def prop(self):
+        return self._property
+
+    @property
+    def range(self):
+        return self._range
+
     def get_range(self) -> _R:
         return self._range
 
@@ -1278,6 +1310,10 @@ class OWLObjectPropertyDomainAxiom(OWLPropertyDomainAxiom[OWLObjectPropertyExpre
                  annotations: Optional[Iterable[OWLAnnotation]] = None):
         super().__init__(property_=property_, domain=domain, annotations=annotations)
 
+    @property
+    def prop(self):
+        return self._property
+
 
 class OWLDataPropertyDomainAxiom(OWLPropertyDomainAxiom[OWLDataPropertyExpression]):
     """ A data property domain axiom DataPropertyDomain( DPE CE ) states that the domain of the
