@@ -34,7 +34,7 @@ from owlapy.owl_individual import OWLNamedIndividual
 from owlapy.owl_literal import IntegerOWLDatatype, BooleanOWLDatatype, DoubleOWLDatatype, OWLLiteral
 from owlapy.owl_property import OWLObjectProperty, OWLDataProperty
 from owlapy.providers import owl_datatype_min_max_exclusive_restriction
-from owlapy.util import NNF
+from owlapy.utils import NNF
 
 from owlapy.owl_data_ranges import OWLDataComplementOf, OWLDataIntersectionOf, OWLDataUnionOf
 from owlapy.class_expression import OWLObjectHasValue, OWLObjectOneOf
